/**
 * Package that contains annotations specific to JSON dataformat.
 *<p>
 * NOTE: starting with version 2.2, general-purpose annotations
 * will be moved to a shared package, and this package will only
 * contains JSON-specific annotations.
 */
package com.fasterxml.jackson.jaxrs.json.annotation;
